@@ -24,6 +24,7 @@
 #include "MediaInfo/Audio/File_Dts.h"
 #include "ZenLib/Utils.h"
 #include "ZenLib/BitStream.h"
+#include "MediaInfo/MediaInfo_Config_MediaInfo.h"
 #if MEDIAINFO_EVENTS
     #include "MediaInfo/MediaInfo_Events.h"
 #endif //MEDIAINFO_EVENTS
@@ -119,7 +120,7 @@ const char*  DTS_ChannelPositions2[16]=
 static const char*  DTS_ChannelLayout[16]=
 {
     "C",
-    "1+1",
+    "M M",
     "L R",
     "L R",
     "Lt Rt",
@@ -289,6 +290,48 @@ std::string DTS_HD_SpeakerActivityMask2 (int16u SpeakerActivityMask, bool AddCs=
     return Text;
 }
 
+//---------------------------------------------------------------------------
+std::string DTS_HD_SpeakerActivityMask_ChannelLayout (int16u SpeakerActivityMask, bool AddCs=false, bool AddLrsRrs=false)
+{
+    std::string Text;
+    if (SpeakerActivityMask&0x0001)
+        Text+=" C";
+    if (SpeakerActivityMask&0x0002)
+        Text+=" L R";
+    if (SpeakerActivityMask&0x0004)
+        Text+=" Ls Rs";
+    if (SpeakerActivityMask&0x0008)
+        Text+=" LFE";
+    if (SpeakerActivityMask&0x0010 || AddCs)
+        Text+=" Cs";
+    if (SpeakerActivityMask&0x0020)
+        Text+=" Lh Rh";
+    if (SpeakerActivityMask&0x0040 || AddLrsRrs)
+        Text+=" Lsr Rsr";
+    if (SpeakerActivityMask&0x0080)
+        Text+=" Ch";
+    if (SpeakerActivityMask&0x0100)
+        Text+=" Oh";
+    if (SpeakerActivityMask&0x0200)
+        Text+=" Lc Rc";
+    if (SpeakerActivityMask&0x0400)
+        Text+=" Lw Rw";
+    if (SpeakerActivityMask&0x0800)
+        Text+=" Lss Rss";
+    if (SpeakerActivityMask&0x1000)
+        Text+=" LFE2";
+    if (SpeakerActivityMask&0x2000)
+        Text+=" Lhs Rhs";
+    if (SpeakerActivityMask&0x4000)
+        Text+=" Chr";
+    if (SpeakerActivityMask&0x8000)
+        Text+=" Lhr";
+
+    if (!Text.empty())
+        Text.erase(0, 1);
+    return Text;
+}
+
 //---------------------------------------------------------------------------
 static const char* DTS_HD_TypeDescriptor[]=
 {
@@ -354,7 +397,7 @@ File_Dts::File_Dts()
     IsRawStream=true;
 
     //In
-    Frame_Count_Valid=MediaInfoLib::Config.ParseSpeed_Get()>=0.3?32:2;
+    Frame_Count_Valid=0;
 
     //Buffer
     Save_Buffer=NULL;
@@ -432,7 +475,7 @@ void File_Dts::Streams_Fill_Extension()
     {
         Data[ChannelPositions].push_back(DTS_HD_SpeakerActivityMask(HD_SpeakerActivityMask, AddCs).c_str()); //AddLrsRrs
         Data[ChannelPositions2].push_back(DTS_HD_SpeakerActivityMask2(HD_SpeakerActivityMask, AddCs).c_str()); //AddLrsRrs
-        Data[ChannelLayout].push_back(Ztring());
+        Data[ChannelLayout].push_back(DTS_HD_SpeakerActivityMask_ChannelLayout(HD_SpeakerActivityMask, AddCs).c_str()); //AddLrsRrs
     }
     else
     {
@@ -490,7 +533,10 @@ void File_Dts::Streams_Fill_Core_ES()
     }
     else
     {
-        Ztring Debug=__T("Debug, Core_Core_AMODE=")+Ztring::ToZtring(Core_Core_AMODE)+(Presence[presence_Core_XCh]?(__T(", Core_XCh_AMODE=")+Ztring::ToZtring(Core_XCh_AMODE)):__T(""))+__T(", Core_Core_LFF=")+Ztring::ToZtring(Core_Core_LFF);
+        Ztring Debug=__T("Debug, Core_Core_AMODE=")+Ztring::ToZtring(Core_Core_AMODE);
+        if(Presence[presence_Core_XCh])
+           Debug +=__T(", Core_XCh_AMODE=")+Ztring::ToZtring(Core_XCh_AMODE);
+        Debug +=__T(", Core_Core_LFF=")+Ztring::ToZtring(Core_Core_LFF);
         Data[ChannelPositions].push_back(Debug);
         Data[ChannelPositions2].push_back(Debug);
         Data[ChannelLayout].push_back(Debug);
@@ -511,9 +557,9 @@ void File_Dts::Streams_Fill_Core()
     if (Core_Core_AMODE<16)
     {
         Data[Channels].push_back(Ztring::ToZtring(DTS_Channels[Core_Core_AMODE]+(Core_Core_LFF?1:0)));
-        Data[ChannelPositions].push_back(Ztring().From_Local(DTS_ChannelPositions[Core_Core_AMODE])+(Core_Core_LFF?__T(", LFE"):__T("")));
-        Data[ChannelPositions2].push_back(Ztring().From_Local(DTS_ChannelPositions2[Core_Core_AMODE])+(Core_Core_LFF?__T(".1"):__T(".0")));
-        Data[ChannelLayout].push_back(Ztring().From_Local(DTS_ChannelLayout[Core_Core_AMODE])+(Core_Core_LFF?__T(" LFE"):__T("")));
+        Data[ChannelPositions].push_back(Ztring().From_UTF8(DTS_ChannelPositions[Core_Core_AMODE])+(Core_Core_LFF?__T(", LFE"):__T("")));
+        Data[ChannelPositions2].push_back(Ztring().From_UTF8(DTS_ChannelPositions2[Core_Core_AMODE])+(Core_Core_LFF?__T(".1"):__T(".0")));
+        Data[ChannelLayout].push_back(Ztring().From_UTF8(DTS_ChannelLayout[Core_Core_AMODE])+(Core_Core_LFF?__T(" LFE"):__T("")));
     }
     else
     {
@@ -657,19 +703,20 @@ void File_Dts::Streams_Fill()
         Data[Profiles].clear(); //Core is the default one
 
     // Filling
+    bool LegacyStreamDisplay=MediaInfoLib::Config.LegacyStreamDisplay_Get();
     Fill(Stream_Audio, 0, Audio_Format_Profile, Data[Profiles].Read());
     Fill(Stream_Audio, 0, Audio_Codec, (Data[Profiles].Find(__T("MA"))!=string::npos || Data[Profiles].Find(__T("HRA"))!=string::npos)?"DTS-HD":"DTS");
     Fill(Stream_General, 0, General_Format_Profile, Retrieve(Stream_Audio, 0, Audio_Format_Profile));
-    Fill(Stream_Audio, 0, Audio_Channel_s_, Data[Channels].Read());
-    Fill(Stream_Audio, 0, Audio_ChannelPositions, Data[ChannelPositions].Read());
-    Fill(Stream_Audio, 0, Audio_ChannelPositions_String2, Data[ChannelPositions2].Read());
-    Fill(Stream_Audio, 0, Audio_ChannelLayout, Data[ChannelLayout].Read());
-    Fill(Stream_Audio, 0, Audio_BitDepth, Data[BitDepth].Read());
-    Fill(Stream_Audio, 0, Audio_SamplingRate, Data[SamplingRate].Read());
-    Fill(Stream_Audio, 0, Audio_BitRate, Data[BitRate].Read());
-    Fill(Stream_Audio, 0, Audio_BitRate_Mode, Data[BitRate_Mode].Read());
+    Fill(Stream_Audio, 0, Audio_Channel_s_, LegacyStreamDisplay?Data[Channels].Read():Data[Channels].Read(0));
+    Fill(Stream_Audio, 0, Audio_ChannelPositions, LegacyStreamDisplay?Data[ChannelPositions].Read():Data[ChannelPositions].Read(0));
+    Fill(Stream_Audio, 0, Audio_ChannelPositions_String2, LegacyStreamDisplay?Data[ChannelPositions2].Read():Data[ChannelPositions2].Read(0));
+    Fill(Stream_Audio, 0, Audio_ChannelLayout, LegacyStreamDisplay?Data[ChannelLayout].Read():Data[ChannelLayout].Read(0));
+    Fill(Stream_Audio, 0, Audio_BitDepth, LegacyStreamDisplay?Data[BitDepth].Read():Data[BitDepth].Read(0));
+    Fill(Stream_Audio, 0, Audio_SamplingRate, LegacyStreamDisplay?Data[SamplingRate].Read():Data[SamplingRate].Read(0));
+    Fill(Stream_Audio, 0, Audio_BitRate, LegacyStreamDisplay?Data[BitRate].Read():(Data[BitRate].Read(0)==__T("Unknown")?Ztring():Data[BitRate].Read(0)));
+    Fill(Stream_Audio, 0, Audio_BitRate_Mode, LegacyStreamDisplay?Data[BitRate_Mode].Read():Data[BitRate_Mode].Read(0));
     Fill(Stream_General, 0, General_OverallBitRate_Mode, Retrieve(Stream_Audio, 0, Audio_BitRate_Mode));
-    Fill(Stream_Audio, 0, Audio_Compression_Mode, Data[Compression_Mode].Read(), true);
+    Fill(Stream_Audio, 0, Audio_Compression_Mode, LegacyStreamDisplay?Data[Compression_Mode].Read():Data[Compression_Mode].Read(0), true);
 
     // Cleanup up
     for (size_t Pos=0; Pos<10; ++Pos)
@@ -689,7 +736,10 @@ void File_Dts::Streams_Finish()
     if (FrameInfo.PTS!=(int64u)-1 && FrameInfo.PTS>PTS_Begin)
     {
         Fill(Stream_Audio, 0, Audio_Duration, ((float64)(FrameInfo.PTS-PTS_Begin))/1000000, 0, true);
-        Fill(Stream_Audio, 0, Audio_FrameCount, ((float64)(FrameInfo.PTS-PTS_Begin))/1000000/32, 0, true);
+        float64 SamplesPerFrame=Retrieve(Stream_Audio, 0, Audio_SamplesPerFrame).To_float64();
+        float64 SamplingRate=Retrieve(Stream_Audio, 0, Audio_SamplingRate).To_float64();
+        if (SamplesPerFrame && SamplingRate)
+            Fill(Stream_Audio, 0, Audio_FrameCount, ((float64)(FrameInfo.PTS-PTS_Begin))/1000000000/(SamplesPerFrame/SamplingRate), 0, true);
     }
 }
 
@@ -715,6 +765,8 @@ bool File_Dts::FileHeader_Begin()
     }
 
     //All should be OK...
+    if (!Frame_Count_Valid)
+        Frame_Count_Valid=Config->ParseSpeed>=0.3?32:2;
     return true;
 }
 
@@ -1084,7 +1136,7 @@ void File_Dts::Data_Parse()
             if (FrameInfo.DTS!=(int64u)-1)
                 FrameInfo.DTS+=FrameInfo.DUR;
             if (FrameInfo.PTS!=(int64u)-1)
-                FrameInfo.PTS=FrameInfo.DTS;
+                FrameInfo.PTS+=FrameInfo.DUR;
         }
     }
 
@@ -1205,7 +1257,7 @@ void File_Dts::Core()
             Fill("DTS");
 
             //No more need data
-            if (!IsSub && MediaInfoLib::Config.ParseSpeed_Get()<1)
+            if (!IsSub && Config->ParseSpeed<1.0)
                 Finish("DTS");
         }
     FILLING_END();
@@ -1329,7 +1381,7 @@ void File_Dts::HD()
             Fill("DTS");
 
             //No more need data
-            if (!IsSub && MediaInfoLib::Config.ParseSpeed_Get()<1)
+            if (!IsSub && Config->ParseSpeed<1.0)
                 Finish("DTS");
         }
     FILLING_END();
@@ -1440,7 +1492,7 @@ void File_Dts::HD_XBR(int64u Size)
 //---------------------------------------------------------------------------
 void File_Dts::HD_XSA(int64u Size)
 {
-    Element_Name("XSA (low bitrate)");
+    Element_Name("LBR (low bitrate)");
     Skip_XX(Size,                                               "Data");
 
     FILLING_BEGIN();
@@ -1550,6 +1602,9 @@ bool File_Dts::FrameSynchPoint_Test()
         {
             if (Word)
             {
+                if (Buffer_Offset+8>Buffer_Size)
+                    return false; //Need more data
+
                 if (BigEndian)
                     Size=((Buffer[Buffer_Offset+5]&0x03)<<12)
                        | ( Buffer[Buffer_Offset+6]      << 4)
@@ -1563,6 +1618,9 @@ bool File_Dts::FrameSynchPoint_Test()
             }
             else
             {
+                if (Buffer_Offset+10>Buffer_Size)
+                    return false; //Need more data
+
                 if (BigEndian)
                     Size=((Buffer[Buffer_Offset+6]&0x03)<<12)
                        | ( Buffer[Buffer_Offset+7]      << 4)
@@ -1575,7 +1633,7 @@ bool File_Dts::FrameSynchPoint_Test()
                 Original_Size=Size*16/14;
             }
         }
-        if (Buffer_Offset+Size>Buffer_Size)
+        if (Buffer_Offset+Original_Size>Buffer_Size)
             return false; //Need more data
 
         if (!Word || !BigEndian)
