@@ -23,6 +23,7 @@ namespace MediaInfoLib
 enum generic
 {
     Generic_Format,
+    Generic_Format_String,
     Generic_Format_Info,
     Generic_Format_Url,
     Generic_Format_Commercial,
@@ -33,6 +34,7 @@ enum generic
     Generic_Format_Tier,
     Generic_Format_Compression,
     Generic_Format_Settings,
+    Generic_Format_AdditionalFeatures,
     Generic_InternetMediaType,
     Generic_CodecID,
     Generic_CodecID_Info,
@@ -204,10 +206,12 @@ enum general
     General_Menu_Language_List,
     General_CompleteName,
     General_FolderName,
+    General_FileNameExtension,
     General_FileName,
     General_FileExtension,
     General_CompleteName_Last,
     General_FolderName_Last,
+    General_FileNameExtension_Last,
     General_FileName_Last,
     General_FileExtension_Last,
     General_Format,
@@ -222,6 +226,7 @@ enum general
     General_Format_Level,
     General_Format_Compression,
     General_Format_Settings,
+    General_Format_AdditionalFeatures,
     General_InternetMediaType,
     General_CodecID,
     General_CodecID_String,
@@ -505,6 +510,7 @@ enum video
     Video_MenuID,
     Video_MenuID_String,
     Video_Format,
+    Video_Format_String,
     Video_Format_Info,
     Video_Format_Url,
     Video_Format_Commercial,
@@ -514,12 +520,14 @@ enum video
     Video_Format_Level,
     Video_Format_Tier,
     Video_Format_Compression,
+    Video_Format_AdditionalFeatures,
     Video_MultiView_BaseProfile,
     Video_MultiView_Count,
     Video_MultiView_Layout,
     Video_DolbyVision_String,
     Video_DolbyVision_Version,
     Video_DolbyVision_Profile,
+    Video_DolbyVision_Level,
     Video_DolbyVision_Layers,
     Video_Format_Settings,
     Video_Format_Settings_BVOP,
@@ -536,6 +544,8 @@ enum video
     Video_Format_Settings_RefFrames,
     Video_Format_Settings_RefFrames_String,
     Video_Format_Settings_Pulldown,
+    Video_Format_Settings_Endianness,
+    Video_Format_Settings_Packing,
     Video_Format_Settings_FrameMode,
     Video_Format_Settings_GOP,
     Video_Format_Settings_PictureStructure,
@@ -819,15 +829,42 @@ enum video
     Video_Tagged_Date,
     Video_Encryption,
     Video_BufferSize,
-    Video_colour_range,
     Video_colour_description_present,
-    Video_colour_primaries,
-    Video_transfer_characteristics,
-    Video_matrix_coefficients,
+    Video_colour_description_present_Source,
     Video_colour_description_present_Original,
+    Video_colour_description_present_Original_Source,
+    Video_colour_range,
+    Video_colour_range_Source,
+    Video_colour_range_Original,
+    Video_colour_range_Original_Source,
+    Video_colour_primaries,
+    Video_colour_primaries_Source,
     Video_colour_primaries_Original,
+    Video_colour_primaries_Original_Source,
+    Video_transfer_characteristics,
+    Video_transfer_characteristics_Source,
     Video_transfer_characteristics_Original,
+    Video_transfer_characteristics_Original_Source,
+    Video_matrix_coefficients,
+    Video_matrix_coefficients_Source,
     Video_matrix_coefficients_Original,
+    Video_matrix_coefficients_Original_Source,
+    Video_MasteringDisplay_ColorPrimaries,
+    Video_MasteringDisplay_ColorPrimaries_Source,
+    Video_MasteringDisplay_ColorPrimaries_Original,
+    Video_MasteringDisplay_ColorPrimaries_Original_Source,
+    Video_MasteringDisplay_Luminance,
+    Video_MasteringDisplay_Luminance_Source,
+    Video_MasteringDisplay_Luminance_Original,
+    Video_MasteringDisplay_Luminance_Original_Source,
+    Video_MaxCLL,
+    Video_MaxCLL_Source,
+    Video_MaxCLL_Original,
+    Video_MaxCLL_Original_Source,
+    Video_MaxFALL,
+    Video_MaxFALL_Source,
+    Video_MaxFALL_Original,
+    Video_MaxFALL_Original_Source,
 };
 
 //---------------------------------------------------------------------------
@@ -852,6 +889,7 @@ enum audio
     Audio_MenuID,
     Audio_MenuID_String,
     Audio_Format,
+    Audio_Format_String,
     Audio_Format_Info,
     Audio_Format_Url,
     Audio_Format_Commercial,
@@ -875,6 +913,7 @@ enum audio
     Audio_Format_Settings_Law,
     Audio_Format_Settings_ITU,
     Audio_Format_Settings_Wrapping,
+    Audio_Format_AdditionalFeatures,
     Audio_Matrix_Format,
     Audio_InternetMediaType,
     Audio_MuxingMode,
@@ -1132,6 +1171,7 @@ enum text
     Text_MenuID,
     Text_MenuID_String,
     Text_Format,
+    Text_Format_String,
     Text_Format_Info,
     Text_Format_Url,
     Text_Format_Commercial,
@@ -1141,6 +1181,7 @@ enum text
     Text_Format_Compression,
     Text_Format_Settings,
     Text_Format_Settings_Wrapping,
+    Text_Format_AdditionalFeatures,
     Text_InternetMediaType,
     Text_MuxingMode,
     Text_MuxingMode_MoreInfo,
@@ -1374,6 +1415,7 @@ enum other
     Other_MenuID_String,
     Other_Type,
     Other_Format,
+    Other_Format_String,
     Other_Format_Info,
     Other_Format_Url,
     Other_Format_Commercial,
@@ -1382,6 +1424,7 @@ enum other
     Other_Format_Profile,
     Other_Format_Compression,
     Other_Format_Settings,
+    Other_Format_AdditionalFeatures,
     Other_MuxingMode,
     Other_CodecID,
     Other_CodecID_String,
@@ -1492,15 +1535,19 @@ enum image
     Image_MenuID_String,
     Image_Title,
     Image_Format,
+    Image_Format_String,
     Image_Format_Info,
     Image_Format_Url,
     Image_Format_Commercial,
     Image_Format_Commercial_IfAny,
     Image_Format_Version,
     Image_Format_Profile,
+    Image_Format_Settings_Endianness,
+    Image_Format_Settings_Packing,
     Image_Format_Compression,
     Image_Format_Settings,
     Image_Format_Settings_Wrapping,
+    Image_Format_AdditionalFeatures,
     Image_InternetMediaType,
     Image_CodecID,
     Image_CodecID_String,
@@ -1616,6 +1663,7 @@ enum menu
     Menu_MenuID,
     Menu_MenuID_String,
     Menu_Format,
+    Menu_Format_String,
     Menu_Format_Info,
     Menu_Format_Url,
     Menu_Format_Commercial,
@@ -1624,6 +1672,7 @@ enum menu
     Menu_Format_Profile,
     Menu_Format_Compression,
     Menu_Format_Settings,
+    Menu_Format_AdditionalFeatures,
     Menu_CodecID,
     Menu_CodecID_String,
     Menu_CodecID_Info,
