@@ -216,6 +216,12 @@
 #if defined(MEDIAINFO_CAF_YES)
     #include "MediaInfo/Audio/File_Caf.h"
 #endif
+#if defined(MEDIAINFO_DSF_YES)
+    #include "MediaInfo/Audio/File_Dsf.h"
+#endif
+#if defined(MEDIAINFO_DSDIFF_YES)
+    #include "MediaInfo/Audio/File_Dsdiff.h"
+#endif
 #if defined(MEDIAINFO_DTS_YES)
     #include "MediaInfo/Audio/File_Dts.h"
 #endif
@@ -598,6 +604,12 @@ bool MediaInfo_Internal::SelectFromExtension (const String &Parser)
     #if defined(MEDIAINFO_CAF_YES)
         else if (Parser==__T("Caf"))          Info=new File_Caf();
     #endif
+    #if defined(MEDIAINFO_DSF_YES)
+        else if (Parser==__T("Dsf"))         Info=new File_Dsf();
+    #endif
+    #if defined(MEDIAINFO_DTS_YES)
+        else if (Parser==__T("Dsdiff"))      Info=new File_Dsdiff();
+    #endif
     #if defined(MEDIAINFO_DTS_YES)
         else if (Parser==__T("Dts"))         Info=new File_Dts();
     #endif
@@ -970,6 +982,12 @@ int MediaInfo_Internal::ListFormats(const String &File_Name)
     #if defined(MEDIAINFO_CAF_YES)
         delete Info; Info=new File_Caf();                if (((Reader_File*)Reader)->Format_Test_PerParser(this, File_Name)>0) return 1;
     #endif
+    #if defined(MEDIAINFO_DSF_YES)
+        delete Info; Info=new File_Dsf();                if (((Reader_File*)Reader)->Format_Test_PerParser(this, File_Name)>0) return 1;
+    #endif
+    #if defined(MEDIAINFO_DSDIFF_YES)
+        delete Info; Info=new File_Dsdiff();             if (((Reader_File*)Reader)->Format_Test_PerParser(this, File_Name)>0) return 1;
+    #endif
     #if defined(MEDIAINFO_DTS_YES)
         delete Info; Info=new File_Dts();                if (((Reader_File*)Reader)->Format_Test_PerParser(this, File_Name)>0) return 1;
     #endif
