@@ -24,13 +24,16 @@
 #include "MediaInfo/Multiple/File_DcpCpl.h"
 #include "MediaInfo/Multiple/File_DcpAm.h"
 #include "MediaInfo/MediaInfo.h"
+#include "MediaInfo/MediaInfo_Config_MediaInfo.h"
 #include "MediaInfo/Multiple/File__ReferenceFilesHelper.h"
 #include "MediaInfo/XmlUtils.h"
-#include "ZenLib/Dir.h"
+#if defined(MEDIAINFO_REFERENCES_YES)
 #include "ZenLib/File.h"
+#endif //defined(MEDIAINFO_REFERENCES_YES)
 #include "ZenLib/FileName.h"
 #include "tinyxml2.h"
 #include <list>
+#include <vector>
 using namespace tinyxml2;
 using namespace std;
 //---------------------------------------------------------------------------
@@ -58,46 +61,10 @@ File_DcpCpl::File_DcpCpl()
         Demux_EventWasSent_Accept_Specific=true;
     #endif //MEDIAINFO_DEMUX
 
-    //Temp
-    ReferenceFiles=NULL;
     //PKL
     PKL_Pos = (size_t)-1;
 }
 
-//---------------------------------------------------------------------------
-File_DcpCpl::~File_DcpCpl()
-{
-    delete ReferenceFiles; //ReferenceFiles=NULL;
-}
-
-//***************************************************************************
-// Streams management
-//***************************************************************************
-
-//---------------------------------------------------------------------------
-void File_DcpCpl::Streams_Finish()
-{
-    if (ReferenceFiles==NULL)
-        return;
-
-    ReferenceFiles->ParseReferences();
-}
-
-//***************************************************************************
-// Buffer - Global
-//***************************************************************************
-
-//---------------------------------------------------------------------------
-#if MEDIAINFO_SEEK
-size_t File_DcpCpl::Read_Buffer_Seek (size_t Method, int64u Value, int64u ID)
-{
-    if (ReferenceFiles==NULL)
-        return 0;
-
-    return ReferenceFiles->Seek(Method, Value, ID);
-}
-#endif //MEDIAINFO_SEEK
-
 //---------------------------------------------------------------------------
 
 static bool IsSmpteSt2067_2(const char* Ns)
@@ -114,6 +81,7 @@ static bool IsSmpteSt2067_3(const char* Ns)
 {
     return Ns &&
            (!strcmp(Ns, "http://www.smpte-ra.org/schemas/2067-3/2013") ||
+            !strcmp(Ns, "http://www.smpte-ra.org/schemas/2067-3/2016") ||
             !strcmp(Ns, "http://www.smpte-ra.org/schemas/2067-3/XXXX")); //Some muxers use XXXX instead of year
 
 }
@@ -155,10 +123,13 @@ bool File_DcpCpl::FileHeader_Begin()
 
     Accept("DcpCpl");
     Fill(Stream_General, 0, General_Format, IsDcp?"DCP CPL":"IMF CPL");
+    #if defined(MEDIAINFO_REFERENCES_YES)
     Config->File_ID_OnlyRoot_Set(false);
 
-    ReferenceFiles=new File__ReferenceFilesHelper(this, Config);
+    ReferenceFiles_Accept(this, Config);
 
+    vector<size_t> AssetCountPerReel;
+                        
     //Parsing main elements
     for (XMLElement* CompositionPlaylist_Item=Root->FirstChildElement(); CompositionPlaylist_Item; CompositionPlaylist_Item=CompositionPlaylist_Item->NextSiblingElement())
     {
@@ -224,9 +195,9 @@ bool File_DcpCpl::FileHeader_Begin()
             {
                 for (XMLElement* EssenceDescriptorList_Item=CompositionPlaylist_Item->FirstChildElement(); EssenceDescriptorList_Item; EssenceDescriptorList_Item=EssenceDescriptorList_Item->NextSiblingElement())
                 {
-                const char* EssenceDescriptorList_Item_Value=EssenceDescriptorList_Item->Value();
-                if (!EssenceDescriptorList_Item_Value)
-                    continue;
+                    const char* EssenceDescriptorList_Item_Value=EssenceDescriptorList_Item->Value();
+                    if (!EssenceDescriptorList_Item_Value)
+                        continue;
 
                     //TimecodeDropFrame
                     if (!strcmp(EssenceDescriptorList_Item_Value, "EssenceDescriptor"))
@@ -307,6 +278,8 @@ bool File_DcpCpl::FileHeader_Begin()
                 //Reel
                 if (MatchQName(ReelList_Item, IsDcp?"Reel":"Segment", NameSpace))
                 {
+                    size_t AssetCount=0;
+
                     for (XMLElement* Reel_Item=ReelList_Item->FirstChildElement(); Reel_Item; Reel_Item=Reel_Item->NextSiblingElement())
                     {
                         //AssetList
@@ -331,6 +304,8 @@ bool File_DcpCpl::FileHeader_Begin()
                                             Sequence->StreamKind=Stream_Video;
                                         else if (!strcmp(AlItemName, "MainSound"))
                                             Sequence->StreamKind=Stream_Audio;
+                                        else if (!strcmp(AlItemName, "MainSubtitle"))
+                                            Sequence->StreamKind=Stream_Text;
                                     }
                                     else if (IsImf && IsSmpteSt2067_2(AlItemNs))
                                     {
@@ -422,16 +397,20 @@ bool File_DcpCpl::FileHeader_Begin()
                                     }
                                     Sequence->StreamID=ReferenceFiles->Sequences_Size()+1;
                                     ReferenceFiles->AddSequence(Sequence);
+
+                                    AssetCount++;
                                 }
                             }
                         }
                     }
+
+                    AssetCountPerReel.push_back(AssetCount);
                 }
             }
         }
     }
 
-    Element_Offset=File_Size;
+    ReferenceFiles->DetectSameReels(AssetCountPerReel);
 
     //Getting files names
     FileName Directory(File_Name);
@@ -476,7 +455,10 @@ bool File_DcpCpl::FileHeader_Begin()
                 ReferenceFiles->UpdateMetaDataFromSourceEncoding(EssenceDescriptor->first, "Format_Profile", Jpeg2000_Rsiz((*SubDescriptor)->Jpeg2000_Rsiz));
     #endif //MEDIAINFO_ADVANCED
 
+    #endif //MEDIAINFO_REFERENCES_YES
+
     //All should be OK...
+    Element_Offset=File_Size;
     return true;
 }
 
@@ -485,12 +467,14 @@ bool File_DcpCpl::FileHeader_Begin()
 //***************************************************************************
 
 //---------------------------------------------------------------------------
+#if defined(MEDIAINFO_REFERENCES_YES)
 void File_DcpCpl::MergeFromAm (File_DcpPkl::streams &StreamsToMerge)
 {
     for (File_DcpPkl::streams::iterator StreamToMerge=StreamsToMerge.begin(); StreamToMerge!=StreamsToMerge.end(); ++StreamToMerge)
         if (!StreamToMerge->ChunkList.empty()) // Note: ChunkLists with more than 1 file are not yet supported)
             ReferenceFiles->UpdateFileName(Ztring().From_UTF8(StreamToMerge->Id), Ztring().From_UTF8(StreamToMerge->ChunkList[0].Path));
 }
+#endif //MEDIAINFO_REFERENCES_YES
 
 } //NameSpace
 
