@@ -235,8 +235,8 @@ private :
             AnnexB_Buffer_Size(0),
             #endif //MEDIAINFO_DEMUX
             profile_space(profile_space_),
-            profile_idc(profile_idc_),
             tier_flag(tier_flag_),
+            profile_idc(profile_idc_),
             level_idc(level_idc_),
             pic_width_in_luma_samples(pic_width_in_luma_samples_),
             pic_height_in_luma_samples(pic_height_in_luma_samples_),
@@ -356,13 +356,16 @@ private :
     void sei_message_buffering_period(int32u &seq_parameter_set_id, int32u payloadSize);
     void sei_message_buffering_period_xxl(seq_parameter_set_struct::vui_parameters_struct::xxl_common* xxL_Common, bool irap_cpb_params_present_flag, seq_parameter_set_struct::vui_parameters_struct::xxl* xxl);
     void sei_message_pic_timing(int32u &seq_parameter_set_id, int32u payloadSize);
+    void sei_message_user_data_registered_itu_t_t35();
     void sei_message_user_data_unregistered(int32u payloadSize);
     void sei_message_user_data_unregistered_Ateme(int32u payloadSize);
     void sei_message_user_data_unregistered_x265(int32u payloadSize);
+    void sei_message_recovery_point();
     void sei_message_active_parameter_sets();
     void sei_message_decoded_picture_hash(int32u payloadSize);
     void sei_message_mastering_display_colour_volume();
     void sei_message_light_level();
+    void sei_alternative_transfer_characteristics();
 
     //Packets - SubElements
     void slice_segment_header();
@@ -399,6 +402,8 @@ private :
     seq_parameter_set_structs           seq_parameter_sets;
     pic_parameter_set_structs           pic_parameter_sets;
 
+    void Clean_Seq_Parameter();
+
     //File specific
     int8u                               lengthSizeMinusOne;
 
@@ -413,6 +418,7 @@ private :
     Ztring                              Encoded_Library_Settings;
     Ztring                              MasteringDisplay_ColorPrimaries;
     Ztring                              MasteringDisplay_Luminance;
+    Ztring                              EtsiTS103433;
     int32u  chroma_format_idc;
     int32u  slice_pic_parameter_set_id;
     int32u  slice_type;
@@ -424,6 +430,7 @@ private :
     int8u   profile_space;
     int8u   profile_idc;
     int8u   level_idc;
+    int8u   preferred_transfer_characteristics;
     bool    tier_flag;
     bool    general_progressive_source_flag;
     bool    general_interlaced_source_flag;
