@@ -32,14 +32,14 @@
     else if (Element_IsOK()) \
     {
 #include <cmath>
+#include "MediaInfo/MediaInfo_Config_MediaInfo.h"
 #if MEDIAINFO_EVENTS
-    #include "MediaInfo/MediaInfo_Config_MediaInfo.h"
     #include "MediaInfo/MediaInfo_Events.h"
     #include "MediaInfo/MediaInfo_Events_Internal.h"
 #endif //MEDIAINFO_EVENTS
 #if MEDIAINFO_DEMUX
     #include <cstring>
-    #include "base64.h"
+    #include "ThirdParty/base64/base64.h"
 #endif //MEDIAINFO_DEMUX
 using namespace std;
 using namespace ZenLib;
@@ -63,7 +63,7 @@ static const char* Vc1_Profile[]=
 };
 
 //---------------------------------------------------------------------------
-static const char* Vc1_ColorimetryFormat[]=
+static const char* Vc1_ChromaSubsamplingFormat[]=
 {
     "",
     "4:2:0",
@@ -164,6 +164,7 @@ const File__Analyze::vlc Vc1_ptype[]=
 };
 
 //---------------------------------------------------------------------------
+/*
 static int32u Vc1_bfraction(int8u Size, int32u Value)
 {
     switch (Size)
@@ -204,6 +205,7 @@ static int32u Vc1_bfraction(int8u Size, int32u Value)
         default: return (int32u)-1;
     }
 };
+*/
 
 //***************************************************************************
 // Constructor/Destructor
@@ -286,7 +288,7 @@ void File_Vc1::Streams_Fill()
     Fill(Stream_Video, 0, Video_Format_Profile, Profile);
     Fill(Stream_Video, 0, Video_Codec_Profile, Profile);
     Fill(Stream_Video, 0, Video_ColorSpace, "YUV");
-    Fill(Stream_Video, 0, Video_Colorimetry, Vc1_ColorimetryFormat[colordiff_format]);
+    Fill(Stream_Video, 0, Video_ChromaSubsampling, Vc1_ChromaSubsamplingFormat[colordiff_format]);
     if (coded_width && coded_height)
     {
         Fill(Stream_Video, StreamPos_Last, Video_Width, (coded_width+1)*2);
@@ -361,10 +363,10 @@ void File_Vc1::Streams_Finish()
                 Numerator=framerateexp+1;
                 Denominator=32;
             }
-            else if (Vc1_FrameRate_dr(frameratecode_dr))
+            else if (const int16u rate_dr=Vc1_FrameRate_dr(frameratecode_dr))
             {
                 Numerator=(int64u)Vc1_FrameRate_enr(frameratecode_enr);
-                Denominator=(int64u)Vc1_FrameRate_dr(frameratecode_dr);
+                Denominator=(int64u)rate_dr;
             }
         }
         if (Numerator)
@@ -530,7 +532,7 @@ bool File_Vc1::Demux_UnpacketizeContainer_Test()
             Demux_Offset++;
         }
 
-        if (Demux_Offset+4>Buffer_Size && File_Offset+Buffer_Size!=File_Size)
+        if (Demux_Offset+4>Buffer_Size && !Config->IsFinishing)
             return false; //No complete frame
 
         if (!Status[IsAccepted])
@@ -562,7 +564,7 @@ bool File_Vc1::Demux_UnpacketizeContainer_Test()
                                 std::string Data_Raw((const char*)(Buffer+Buffer_Offset), (size_t)(Header_End-Buffer_Offset));
                                 std::string Data_Base64(Base64::encode(Data_Raw));
                                 Fill(Stream_Video, StreamPos_Last, "Demux_InitBytes", Data_Base64);
-                                (*Stream_More)[Stream_Video][StreamPos_Last](Ztring().From_Local("Demux_InitBytes"), Info_Options)=__T("N NT");
+                                Fill_SetOptions(Stream_Video, StreamPos_Last, "Demux_InitBytes", "N NT");
                                 }
                                 break;
                     default :   ;
@@ -673,7 +675,7 @@ bool File_Vc1::Header_Parser_Fill_Size()
     //Must wait more data?
     if (Buffer_Offset_Temp+4>Buffer_Size)
     {
-        if (FrameIsAlwaysComplete || File_Offset+Buffer_Size==File_Size)
+        if (FrameIsAlwaysComplete || Config->IsFinishing)
             Buffer_Offset_Temp=Buffer_Size; //We are sure that the next bytes are a start
         else
             return false;
@@ -931,7 +933,7 @@ void File_Vc1::FrameHeader()
         {
             Fill("VC-1");
 
-            if (!IsSub && MediaInfoLib::Config.ParseSpeed_Get()<1)
+            if (!IsSub && Config->ParseSpeed<1.0)
                 Finish("VC-1");
         }
 
@@ -1035,7 +1037,7 @@ void File_Vc1::EntryPointHeader()
                                 std::string Data_Raw((char*)InitData_Buffer_Temp, InitData_Buffer_Temp_Size);
                                 std::string Data_Base64(Base64::encode(Data_Raw));
                                 Fill(Stream_Video, StreamPos_Last, "Demux_InitBytes", Data_Base64);
-                                (*Stream_More)[Stream_Video][StreamPos_Last](Ztring().From_Local("Demux_InitBytes"), Info_Options)=__T("N NT");
+                                Fill_SetOptions(Stream_Video, StreamPos_Last, "Demux_InitBytes", "N NT");
                                 }
                                 break;
                     default :   ;
@@ -1083,7 +1085,7 @@ void File_Vc1::SequenceHeader()
     else if (profile==3) //Advanced
     {
         Get_S1 ( 3, level,                                      "level");
-        Get_S1 ( 2, colordiff_format,                           "colordiff_format"); Param_Info1(Vc1_ColorimetryFormat[colordiff_format]);
+        Get_S1 ( 2, colordiff_format,                           "colordiff_format"); Param_Info1(Vc1_ChromaSubsamplingFormat[colordiff_format]);
         Skip_S1( 3,                                             "frmrtq_postproc");
         Skip_S1( 5,                                             "bitrtq_postproc");
         Skip_SB(                                                "postprocflag");
@@ -1162,8 +1164,8 @@ void File_Vc1::SequenceHeader()
         {
             if (framerate_form)
                 FrameRate=((float64)(framerateexp+1))/(float32)64;
-            else if (Vc1_FrameRate_dr(frameratecode_dr))
-                FrameRate=((float64)Vc1_FrameRate_enr(frameratecode_enr))/Vc1_FrameRate_dr(frameratecode_dr);
+            else if (const int16u rate_dr = Vc1_FrameRate_dr(frameratecode_dr))
+                FrameRate=((float64)Vc1_FrameRate_enr(frameratecode_enr))/rate_dr;
         }
 
         if (From_WMV3)
